# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .geo.geo import (
    GeoResource,
    AsyncGeoResource,
    GeoResourceWithRawResponse,
    AsyncGeoResourceWithRawResponse,
    GeoResourceWithStreamingResponse,
    AsyncGeoResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .recent.recent import (
    RecentResource,
    AsyncRecentResource,
    RecentResourceWithRawResponse,
    AsyncRecentResourceWithRawResponse,
    RecentResourceWithStreamingResponse,
    AsyncRecentResourceWithStreamingResponse,
)
from .nearest.nearest import (
    NearestResource,
    AsyncNearestResource,
    NearestResourceWithRawResponse,
    AsyncNearestResourceWithRawResponse,
    NearestResourceWithStreamingResponse,
    AsyncNearestResourceWithStreamingResponse,
)

__all__ = ["ObservationsResource", "AsyncObservationsResource"]


class ObservationsResource(SyncAPIResource):
    @cached_property
    def recent(self) -> RecentResource:
        return RecentResource(self._client)

    @cached_property
    def geo(self) -> GeoResource:
        return GeoResource(self._client)

    @cached_property
    def nearest(self) -> NearestResource:
        return NearestResource(self._client)

    @cached_property
    def with_raw_response(self) -> ObservationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return ObservationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ObservationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return ObservationsResourceWithStreamingResponse(self)


class AsyncObservationsResource(AsyncAPIResource):
    @cached_property
    def recent(self) -> AsyncRecentResource:
        return AsyncRecentResource(self._client)

    @cached_property
    def geo(self) -> AsyncGeoResource:
        return AsyncGeoResource(self._client)

    @cached_property
    def nearest(self) -> AsyncNearestResource:
        return AsyncNearestResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncObservationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncObservationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncObservationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncObservationsResourceWithStreamingResponse(self)


class ObservationsResourceWithRawResponse:
    def __init__(self, observations: ObservationsResource) -> None:
        self._observations = observations

    @cached_property
    def recent(self) -> RecentResourceWithRawResponse:
        return RecentResourceWithRawResponse(self._observations.recent)

    @cached_property
    def geo(self) -> GeoResourceWithRawResponse:
        return GeoResourceWithRawResponse(self._observations.geo)

    @cached_property
    def nearest(self) -> NearestResourceWithRawResponse:
        return NearestResourceWithRawResponse(self._observations.nearest)


class AsyncObservationsResourceWithRawResponse:
    def __init__(self, observations: AsyncObservationsResource) -> None:
        self._observations = observations

    @cached_property
    def recent(self) -> AsyncRecentResourceWithRawResponse:
        return AsyncRecentResourceWithRawResponse(self._observations.recent)

    @cached_property
    def geo(self) -> AsyncGeoResourceWithRawResponse:
        return AsyncGeoResourceWithRawResponse(self._observations.geo)

    @cached_property
    def nearest(self) -> AsyncNearestResourceWithRawResponse:
        return AsyncNearestResourceWithRawResponse(self._observations.nearest)


class ObservationsResourceWithStreamingResponse:
    def __init__(self, observations: ObservationsResource) -> None:
        self._observations = observations

    @cached_property
    def recent(self) -> RecentResourceWithStreamingResponse:
        return RecentResourceWithStreamingResponse(self._observations.recent)

    @cached_property
    def geo(self) -> GeoResourceWithStreamingResponse:
        return GeoResourceWithStreamingResponse(self._observations.geo)

    @cached_property
    def nearest(self) -> NearestResourceWithStreamingResponse:
        return NearestResourceWithStreamingResponse(self._observations.nearest)


class AsyncObservationsResourceWithStreamingResponse:
    def __init__(self, observations: AsyncObservationsResource) -> None:
        self._observations = observations

    @cached_property
    def recent(self) -> AsyncRecentResourceWithStreamingResponse:
        return AsyncRecentResourceWithStreamingResponse(self._observations.recent)

    @cached_property
    def geo(self) -> AsyncGeoResourceWithStreamingResponse:
        return AsyncGeoResourceWithStreamingResponse(self._observations.geo)

    @cached_property
    def nearest(self) -> AsyncNearestResourceWithStreamingResponse:
        return AsyncNearestResourceWithStreamingResponse(self._observations.nearest)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from phoebe_bird import Phoebe, AsyncPhoebe
from tests.utils import assert_matches_type
from phoebe_bird.types.product import Top100RetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTop100:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Phoebe) -> None:
        top100 = client.product.top100.retrieve(
            d=1,
            region_code="regionCode",
            y=0,
            m=1,
        )
        assert_matches_type(Top100RetrieveResponse, top100, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: Phoebe) -> None:
        top100 = client.product.top100.retrieve(
            d=1,
            region_code="regionCode",
            y=0,
            m=1,
            max_results=1,
            ranked_by="spp",
        )
        assert_matches_type(Top100RetrieveResponse, top100, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Phoebe) -> None:
        response = client.product.top100.with_raw_response.retrieve(
            d=1,
            region_code="regionCode",
            y=0,
            m=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        top100 = response.parse()
        assert_matches_type(Top100RetrieveResponse, top100, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Phoebe) -> None:
        with client.product.top100.with_streaming_response.retrieve(
            d=1,
            region_code="regionCode",
            y=0,
            m=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            top100 = response.parse()
            assert_matches_type(Top100RetrieveResponse, top100, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Phoebe) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `region_code` but received ''"):
            client.product.top100.with_raw_response.retrieve(
                d=1,
                region_code="",
                y=0,
                m=1,
            )


class TestAsyncTop100:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncPhoebe) -> None:
        top100 = await async_client.product.top100.retrieve(
            d=1,
            region_code="regionCode",
            y=0,
            m=1,
        )
        assert_matches_type(Top100RetrieveResponse, top100, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncPhoebe) -> None:
        top100 = await async_client.product.top100.retrieve(
            d=1,
            region_code="regionCode",
            y=0,
            m=1,
            max_results=1,
            ranked_by="spp",
        )
        assert_matches_type(Top100RetrieveResponse, top100, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncPhoebe) -> None:
        response = await async_client.product.top100.with_raw_response.retrieve(
            d=1,
            region_code="regionCode",
            y=0,
            m=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        top100 = await response.parse()
        assert_matches_type(Top100RetrieveResponse, top100, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncPhoebe) -> None:
        async with async_client.product.top100.with_streaming_response.retrieve(
            d=1,
            region_code="regionCode",
            y=0,
            m=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            top100 = await response.parse()
            assert_matches_type(Top100RetrieveResponse, top100, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncPhoebe) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `region_code` but received ''"):
            await async_client.product.top100.with_raw_response.retrieve(
                d=1,
                region_code="",
                y=0,
                m=1,
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "ListRetrieveResponse",
    "ListRetrieveResponseItem",
    "ListRetrieveResponseItemLoc",
    "ListRetrieveResponseItemOb",
    "ListRetrieveResponseItemObObsAux",
]


class ListRetrieveResponseItemLoc(BaseModel):
    country_code: Optional[str] = FieldInfo(alias="countryCode", default=None)

    country_name: Optional[str] = FieldInfo(alias="countryName", default=None)

    hierarchical_name: Optional[str] = FieldInfo(alias="hierarchicalName", default=None)

    is_hotspot: Optional[bool] = FieldInfo(alias="isHotspot", default=None)

    lat: Optional[float] = None

    latitude: Optional[float] = None

    lng: Optional[float] = None

    loc_id: Optional[str] = FieldInfo(alias="locId", default=None)

    loc_name: Optional[str] = FieldInfo(alias="locName", default=None)

    longitude: Optional[float] = None

    name: Optional[str] = None

    subnational1_code: Optional[str] = FieldInfo(alias="subnational1Code", default=None)

    subnational1_name: Optional[str] = FieldInfo(alias="subnational1Name", default=None)


class ListRetrieveResponseItemObObsAux(BaseModel):
    aux_code: Optional[str] = FieldInfo(alias="auxCode", default=None)

    entry_method_code: Optional[str] = FieldInfo(alias="entryMethodCode", default=None)

    field_name: Optional[str] = FieldInfo(alias="fieldName", default=None)

    obs_id: Optional[str] = FieldInfo(alias="obsId", default=None)

    species_code: Optional[str] = FieldInfo(alias="speciesCode", default=None)

    sub_id: Optional[str] = FieldInfo(alias="subId", default=None)

    value: Optional[str] = None


class ListRetrieveResponseItemOb(BaseModel):
    obs_aux: Optional[List[ListRetrieveResponseItemObObsAux]] = FieldInfo(alias="obsAux", default=None)

    obs_dt: Optional[str] = FieldInfo(alias="obsDt", default=None)

    obs_id: Optional[str] = FieldInfo(alias="obsId", default=None)

    species_code: Optional[str] = FieldInfo(alias="speciesCode", default=None)


class ListRetrieveResponseItem(BaseModel):
    all_obs_reported: Optional[bool] = FieldInfo(alias="allObsReported", default=None)

    checklist_id: Optional[str] = FieldInfo(alias="checklistId", default=None)

    creation_dt: Optional[str] = FieldInfo(alias="creationDt", default=None)

    duration_hrs: Optional[float] = FieldInfo(alias="durationHrs", default=None)

    iso_obs_date: Optional[str] = FieldInfo(alias="isoObsDate", default=None)

    last_edited_dt: Optional[str] = FieldInfo(alias="lastEditedDt", default=None)

    loc: Optional[ListRetrieveResponseItemLoc] = None

    loc_id: Optional[str] = FieldInfo(alias="locId", default=None)

    num_observers: Optional[int] = FieldInfo(alias="numObservers", default=None)

    num_species: Optional[int] = FieldInfo(alias="numSpecies", default=None)

    obs: Optional[List[ListRetrieveResponseItemOb]] = None

    obs_dt: Optional[str] = FieldInfo(alias="obsDt", default=None)

    obs_time: Optional[str] = FieldInfo(alias="obsTime", default=None)

    obs_time_valid: Optional[bool] = FieldInfo(alias="obsTimeValid", default=None)

    proj_id: Optional[str] = FieldInfo(alias="projId", default=None)

    protocol_id: Optional[str] = FieldInfo(alias="protocolId", default=None)

    sub_id: Optional[str] = FieldInfo(alias="subId", default=None)

    submission_method_code: Optional[str] = FieldInfo(alias="submissionMethodCode", default=None)

    subnational1_code: Optional[str] = FieldInfo(alias="subnational1Code", default=None)

    user_display_name: Optional[str] = FieldInfo(alias="userDisplayName", default=None)


ListRetrieveResponse: TypeAlias = List[ListRetrieveResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["GeoRetrieveParams"]


class GeoRetrieveParams(TypedDict, total=False):
    lat: Required[float]

    lng: Required[float]

    back: int
    """The number of days back to fetch hotspots."""

    dist: int
    """The search radius from the given position, in kilometers."""

    fmt: Literal["csv", "json"]
    """Fetch the records in CSV or JSON format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .lists import (
    ListsResource,
    AsyncListsResource,
    ListsResourceWithRawResponse,
    AsyncListsResourceWithRawResponse,
    ListsResourceWithStreamingResponse,
    AsyncListsResourceWithStreamingResponse,
)
from .historical import (
    HistoricalResource,
    AsyncHistoricalResource,
    HistoricalResourceWithRawResponse,
    AsyncHistoricalResourceWithRawResponse,
    HistoricalResourceWithStreamingResponse,
    AsyncHistoricalResourceWithStreamingResponse,
)

__all__ = [
    "HistoricalResource",
    "AsyncHistoricalResource",
    "HistoricalResourceWithRawResponse",
    "AsyncHistoricalResourceWithRawResponse",
    "HistoricalResourceWithStreamingResponse",
    "AsyncHistoricalResourceWithStreamingResponse",
    "ListsResource",
    "AsyncListsResource",
    "ListsResourceWithRawResponse",
    "AsyncListsResourceWithRawResponse",
    "ListsResourceWithStreamingResponse",
    "AsyncListsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["Top100RetrieveResponse", "Top100RetrieveResponseItem"]


class Top100RetrieveResponseItem(BaseModel):
    num_complete_checklists: Optional[int] = FieldInfo(alias="numCompleteChecklists", default=None)

    num_species: Optional[int] = FieldInfo(alias="numSpecies", default=None)

    profile_handle: Optional[str] = FieldInfo(alias="profileHandle", default=None)

    row_num: Optional[int] = FieldInfo(alias="rowNum", default=None)

    user_display_name: Optional[str] = FieldInfo(alias="userDisplayName", default=None)

    user_id: Optional[str] = FieldInfo(alias="userId", default=None)


Top100RetrieveResponse: TypeAlias = List[Top100RetrieveResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ref.taxonomy.form_list_response import FormListResponse

__all__ = ["FormsResource", "AsyncFormsResource"]


class FormsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FormsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return FormsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FormsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return FormsResourceWithStreamingResponse(self)

    def list(
        self,
        species_code: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FormListResponse:
        """For a species, get the list of subspecies recognised in the taxonomy.

        The
        results include the species that was passed in.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not species_code:
            raise ValueError(f"Expected a non-empty value for `species_code` but received {species_code!r}")
        return self._get(
            f"/ref/taxon/forms/{species_code}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FormListResponse,
        )


class AsyncFormsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFormsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFormsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFormsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncFormsResourceWithStreamingResponse(self)

    async def list(
        self,
        species_code: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FormListResponse:
        """For a species, get the list of subspecies recognised in the taxonomy.

        The
        results include the species that was passed in.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not species_code:
            raise ValueError(f"Expected a non-empty value for `species_code` but received {species_code!r}")
        return await self._get(
            f"/ref/taxon/forms/{species_code}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FormListResponse,
        )


class FormsResourceWithRawResponse:
    def __init__(self, forms: FormsResource) -> None:
        self._forms = forms

        self.list = to_raw_response_wrapper(
            forms.list,
        )


class AsyncFormsResourceWithRawResponse:
    def __init__(self, forms: AsyncFormsResource) -> None:
        self._forms = forms

        self.list = async_to_raw_response_wrapper(
            forms.list,
        )


class FormsResourceWithStreamingResponse:
    def __init__(self, forms: FormsResource) -> None:
        self._forms = forms

        self.list = to_streamed_response_wrapper(
            forms.list,
        )


class AsyncFormsResourceWithStreamingResponse:
    def __init__(self, forms: AsyncFormsResource) -> None:
        self._forms = forms

        self.list = async_to_streamed_response_wrapper(
            forms.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.product import top100_retrieve_params
from ...types.product.top100_retrieve_response import Top100RetrieveResponse

__all__ = ["Top100Resource", "AsyncTop100Resource"]


class Top100Resource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> Top100ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return Top100ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> Top100ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return Top100ResourceWithStreamingResponse(self)

    def retrieve(
        self,
        d: int,
        *,
        region_code: str,
        y: int,
        m: int,
        max_results: int | NotGiven = NOT_GIVEN,
        ranked_by: Literal["spp", "cl"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Top100RetrieveResponse:
        """
        Get the top 100 contributors on a given date for a country or region.

        #### Notes

        The results are updated every 15 minutes.

        When ordering by the number of completed checklists, the number of species seen
        will always be zero. Similarly when ordering by the number of species seen the
        number of completed checklists will always be zero. <b>Selected Response Field
        Notes</b>

        profileHandle - if a user has enabled their profile, this is the handle to reach
        it via ebird.org/ebird/profile/{profileHandle}

        numSpecies - always zero when checklistSort parameter is true. Invalid
        observations ARE included in this total numCompleteChecklists - always zero when
        checklistSort parameter is false

        Args:
          max_results: Only fetch this number of contributors.

          ranked_by: Order by number of complete checklists (cl) or by number of species seen (spp).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return self._get(
            f"/product/top100/{region_code}/{y}/{m}/{d}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "max_results": max_results,
                        "ranked_by": ranked_by,
                    },
                    top100_retrieve_params.Top100RetrieveParams,
                ),
            ),
            cast_to=Top100RetrieveResponse,
        )


class AsyncTop100Resource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTop100ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTop100ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTop100ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncTop100ResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        d: int,
        *,
        region_code: str,
        y: int,
        m: int,
        max_results: int | NotGiven = NOT_GIVEN,
        ranked_by: Literal["spp", "cl"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Top100RetrieveResponse:
        """
        Get the top 100 contributors on a given date for a country or region.

        #### Notes

        The results are updated every 15 minutes.

        When ordering by the number of completed checklists, the number of species seen
        will always be zero. Similarly when ordering by the number of species seen the
        number of completed checklists will always be zero. <b>Selected Response Field
        Notes</b>

        profileHandle - if a user has enabled their profile, this is the handle to reach
        it via ebird.org/ebird/profile/{profileHandle}

        numSpecies - always zero when checklistSort parameter is true. Invalid
        observations ARE included in this total numCompleteChecklists - always zero when
        checklistSort parameter is false

        Args:
          max_results: Only fetch this number of contributors.

          ranked_by: Order by number of complete checklists (cl) or by number of species seen (spp).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return await self._get(
            f"/product/top100/{region_code}/{y}/{m}/{d}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "max_results": max_results,
                        "ranked_by": ranked_by,
                    },
                    top100_retrieve_params.Top100RetrieveParams,
                ),
            ),
            cast_to=Top100RetrieveResponse,
        )


class Top100ResourceWithRawResponse:
    def __init__(self, top100: Top100Resource) -> None:
        self._top100 = top100

        self.retrieve = to_raw_response_wrapper(
            top100.retrieve,
        )


class AsyncTop100ResourceWithRawResponse:
    def __init__(self, top100: AsyncTop100Resource) -> None:
        self._top100 = top100

        self.retrieve = async_to_raw_response_wrapper(
            top100.retrieve,
        )


class Top100ResourceWithStreamingResponse:
    def __init__(self, top100: Top100Resource) -> None:
        self._top100 = top100

        self.retrieve = to_streamed_response_wrapper(
            top100.retrieve,
        )


class AsyncTop100ResourceWithStreamingResponse:
    def __init__(self, top100: AsyncTop100Resource) -> None:
        self._top100 = top100

        self.retrieve = async_to_streamed_response_wrapper(
            top100.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

__all__ = ["FormListResponse"]

FormListResponse: TypeAlias = List[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ref.taxonomy import species_group_list_params
from ....types.ref.taxonomy.species_group_list_response import SpeciesGroupListResponse

__all__ = ["SpeciesGroupsResource", "AsyncSpeciesGroupsResource"]


class SpeciesGroupsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpeciesGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return SpeciesGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeciesGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return SpeciesGroupsResourceWithStreamingResponse(self)

    def list(
        self,
        species_grouping: Literal["merlin", "ebird"],
        *,
        group_name_locale: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpeciesGroupListResponse:
        """Get the list of species groups, e.g.

        terns, finches, etc. #### Notes Merlin puts
        like birds together, with Falcons next to Hawks, whereas eBird follows taxonomic
        order.

        Args:
          species_grouping: The order in which groups are returned.

          group_name_locale: Locale for species group names. English names are returned for any non-listed
              locale or any non-translated group name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not species_grouping:
            raise ValueError(f"Expected a non-empty value for `species_grouping` but received {species_grouping!r}")
        return self._get(
            f"/ref/sppgroup/{species_grouping}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"group_name_locale": group_name_locale}, species_group_list_params.SpeciesGroupListParams
                ),
            ),
            cast_to=SpeciesGroupListResponse,
        )


class AsyncSpeciesGroupsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpeciesGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeciesGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeciesGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncSpeciesGroupsResourceWithStreamingResponse(self)

    async def list(
        self,
        species_grouping: Literal["merlin", "ebird"],
        *,
        group_name_locale: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpeciesGroupListResponse:
        """Get the list of species groups, e.g.

        terns, finches, etc. #### Notes Merlin puts
        like birds together, with Falcons next to Hawks, whereas eBird follows taxonomic
        order.

        Args:
          species_grouping: The order in which groups are returned.

          group_name_locale: Locale for species group names. English names are returned for any non-listed
              locale or any non-translated group name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not species_grouping:
            raise ValueError(f"Expected a non-empty value for `species_grouping` but received {species_grouping!r}")
        return await self._get(
            f"/ref/sppgroup/{species_grouping}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"group_name_locale": group_name_locale}, species_group_list_params.SpeciesGroupListParams
                ),
            ),
            cast_to=SpeciesGroupListResponse,
        )


class SpeciesGroupsResourceWithRawResponse:
    def __init__(self, species_groups: SpeciesGroupsResource) -> None:
        self._species_groups = species_groups

        self.list = to_raw_response_wrapper(
            species_groups.list,
        )


class AsyncSpeciesGroupsResourceWithRawResponse:
    def __init__(self, species_groups: AsyncSpeciesGroupsResource) -> None:
        self._species_groups = species_groups

        self.list = async_to_raw_response_wrapper(
            species_groups.list,
        )


class SpeciesGroupsResourceWithStreamingResponse:
    def __init__(self, species_groups: SpeciesGroupsResource) -> None:
        self._species_groups = species_groups

        self.list = to_streamed_response_wrapper(
            species_groups.list,
        )


class AsyncSpeciesGroupsResourceWithStreamingResponse:
    def __init__(self, species_groups: AsyncSpeciesGroupsResource) -> None:
        self._species_groups = species_groups

        self.list = async_to_streamed_response_wrapper(
            species_groups.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from ...._models import BaseModel

__all__ = ["ListListResponse", "ListListResponseItem"]


class ListListResponseItem(BaseModel):
    code: Optional[str] = None

    name: Optional[str] = None


ListListResponse: TypeAlias = List[ListListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.product.lists import historical_retrieve_params
from ....types.product.lists.historical_retrieve_response import HistoricalRetrieveResponse

__all__ = ["HistoricalResource", "AsyncHistoricalResource"]


class HistoricalResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HistoricalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return HistoricalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HistoricalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return HistoricalResourceWithStreamingResponse(self)

    def retrieve(
        self,
        d: int,
        *,
        region_code: str,
        y: int,
        m: int,
        max_results: int | NotGiven = NOT_GIVEN,
        sort_key: Literal["obs_dt", "creation_dt"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> HistoricalRetrieveResponse:
        """
        Get information on the checklists submitted on a given date for a country or
        region.

        Args:
          max_results: Only fetch this number of checklists.

          sort_key: Order the results by the date of the checklist or by the date it was submitted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return self._get(
            f"/product/lists/{region_code}/{y}/{m}/{d}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "max_results": max_results,
                        "sort_key": sort_key,
                    },
                    historical_retrieve_params.HistoricalRetrieveParams,
                ),
            ),
            cast_to=HistoricalRetrieveResponse,
        )


class AsyncHistoricalResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHistoricalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHistoricalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHistoricalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncHistoricalResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        d: int,
        *,
        region_code: str,
        y: int,
        m: int,
        max_results: int | NotGiven = NOT_GIVEN,
        sort_key: Literal["obs_dt", "creation_dt"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> HistoricalRetrieveResponse:
        """
        Get information on the checklists submitted on a given date for a country or
        region.

        Args:
          max_results: Only fetch this number of checklists.

          sort_key: Order the results by the date of the checklist or by the date it was submitted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return await self._get(
            f"/product/lists/{region_code}/{y}/{m}/{d}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "max_results": max_results,
                        "sort_key": sort_key,
                    },
                    historical_retrieve_params.HistoricalRetrieveParams,
                ),
            ),
            cast_to=HistoricalRetrieveResponse,
        )


class HistoricalResourceWithRawResponse:
    def __init__(self, historical: HistoricalResource) -> None:
        self._historical = historical

        self.retrieve = to_raw_response_wrapper(
            historical.retrieve,
        )


class AsyncHistoricalResourceWithRawResponse:
    def __init__(self, historical: AsyncHistoricalResource) -> None:
        self._historical = historical

        self.retrieve = async_to_raw_response_wrapper(
            historical.retrieve,
        )


class HistoricalResourceWithStreamingResponse:
    def __init__(self, historical: HistoricalResource) -> None:
        self._historical = historical

        self.retrieve = to_streamed_response_wrapper(
            historical.retrieve,
        )


class AsyncHistoricalResourceWithStreamingResponse:
    def __init__(self, historical: AsyncHistoricalResource) -> None:
        self._historical = historical

        self.retrieve = async_to_streamed_response_wrapper(
            historical.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .recent_list_params import RecentListParams as RecentListParams
from .recent_list_response import RecentListResponse as RecentListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.data.observations.recent import historic_list_params
from .....types.data.observations.recent.historic_list_response import HistoricListResponse

__all__ = ["HistoricResource", "AsyncHistoricResource"]


class HistoricResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HistoricResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return HistoricResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HistoricResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return HistoricResourceWithStreamingResponse(self)

    def list(
        self,
        d: int,
        *,
        region_code: str,
        y: int,
        m: int,
        cat: Literal["species", "slash", "issf", "spuh", "hybrid", "domestic", "form", "intergrade"]
        | NotGiven = NOT_GIVEN,
        detail: Literal["simple", "full"] | NotGiven = NOT_GIVEN,
        hotspot: bool | NotGiven = NOT_GIVEN,
        include_provisional: bool | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        r: List[str] | NotGiven = NOT_GIVEN,
        rank: Literal["mrec", "create"] | NotGiven = NOT_GIVEN,
        spp_locale: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> HistoricListResponse:
        """
        Get a list of all taxa seen in a country, region or location on a specific date,
        with the specific observations determined by the "rank" parameter (defaults to
        latest observation on the date).

        #### Notes Responses may be cached for 30 minutes

        Args:
          cat: Only fetch observations from these taxonomic categories

          detail: Include a subset (simple), or all (full), of the fields available.

          hotspot: Only fetch observations from hotspots

          include_provisional: Include observations which have not yet been reviewed.

          max_results: Only fetch this number of observations

          r: Fetch observations from up to 50 locations

          rank: Include latest observation of the day, or the first added

          spp_locale: Use this language for species common names

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return self._get(
            f"/data/obs/{region_code}/historic/{y}/{m}/{d}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cat": cat,
                        "detail": detail,
                        "hotspot": hotspot,
                        "include_provisional": include_provisional,
                        "max_results": max_results,
                        "r": r,
                        "rank": rank,
                        "spp_locale": spp_locale,
                    },
                    historic_list_params.HistoricListParams,
                ),
            ),
            cast_to=HistoricListResponse,
        )


class AsyncHistoricResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHistoricResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHistoricResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHistoricResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncHistoricResourceWithStreamingResponse(self)

    async def list(
        self,
        d: int,
        *,
        region_code: str,
        y: int,
        m: int,
        cat: Literal["species", "slash", "issf", "spuh", "hybrid", "domestic", "form", "intergrade"]
        | NotGiven = NOT_GIVEN,
        detail: Literal["simple", "full"] | NotGiven = NOT_GIVEN,
        hotspot: bool | NotGiven = NOT_GIVEN,
        include_provisional: bool | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        r: List[str] | NotGiven = NOT_GIVEN,
        rank: Literal["mrec", "create"] | NotGiven = NOT_GIVEN,
        spp_locale: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> HistoricListResponse:
        """
        Get a list of all taxa seen in a country, region or location on a specific date,
        with the specific observations determined by the "rank" parameter (defaults to
        latest observation on the date).

        #### Notes Responses may be cached for 30 minutes

        Args:
          cat: Only fetch observations from these taxonomic categories

          detail: Include a subset (simple), or all (full), of the fields available.

          hotspot: Only fetch observations from hotspots

          include_provisional: Include observations which have not yet been reviewed.

          max_results: Only fetch this number of observations

          r: Fetch observations from up to 50 locations

          rank: Include latest observation of the day, or the first added

          spp_locale: Use this language for species common names

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return await self._get(
            f"/data/obs/{region_code}/historic/{y}/{m}/{d}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cat": cat,
                        "detail": detail,
                        "hotspot": hotspot,
                        "include_provisional": include_provisional,
                        "max_results": max_results,
                        "r": r,
                        "rank": rank,
                        "spp_locale": spp_locale,
                    },
                    historic_list_params.HistoricListParams,
                ),
            ),
            cast_to=HistoricListResponse,
        )


class HistoricResourceWithRawResponse:
    def __init__(self, historic: HistoricResource) -> None:
        self._historic = historic

        self.list = to_raw_response_wrapper(
            historic.list,
        )


class AsyncHistoricResourceWithRawResponse:
    def __init__(self, historic: AsyncHistoricResource) -> None:
        self._historic = historic

        self.list = async_to_raw_response_wrapper(
            historic.list,
        )


class HistoricResourceWithStreamingResponse:
    def __init__(self, historic: HistoricResource) -> None:
        self._historic = historic

        self.list = to_streamed_response_wrapper(
            historic.list,
        )


class AsyncHistoricResourceWithStreamingResponse:
    def __init__(self, historic: AsyncHistoricResource) -> None:
        self._historic = historic

        self.list = async_to_streamed_response_wrapper(
            historic.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.product.species_list_list_response import SpeciesListListResponse

__all__ = ["SpeciesListResource", "AsyncSpeciesListResource"]


class SpeciesListResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpeciesListResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return SpeciesListResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeciesListResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return SpeciesListResourceWithStreamingResponse(self)

    def list(
        self,
        region_code: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpeciesListListResponse:
        """
        Get a list of species codes ever seen in a region, in taxonomic order (species
        taxa only)

        #### Notes The results are usually updated every 10 seconds for locations, every day for larger regions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return self._get(
            f"/product/spplist/{region_code}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeciesListListResponse,
        )


class AsyncSpeciesListResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpeciesListResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeciesListResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeciesListResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncSpeciesListResourceWithStreamingResponse(self)

    async def list(
        self,
        region_code: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpeciesListListResponse:
        """
        Get a list of species codes ever seen in a region, in taxonomic order (species
        taxa only)

        #### Notes The results are usually updated every 10 seconds for locations, every day for larger regions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return await self._get(
            f"/product/spplist/{region_code}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeciesListListResponse,
        )


class SpeciesListResourceWithRawResponse:
    def __init__(self, species_list: SpeciesListResource) -> None:
        self._species_list = species_list

        self.list = to_raw_response_wrapper(
            species_list.list,
        )


class AsyncSpeciesListResourceWithRawResponse:
    def __init__(self, species_list: AsyncSpeciesListResource) -> None:
        self._species_list = species_list

        self.list = async_to_raw_response_wrapper(
            species_list.list,
        )


class SpeciesListResourceWithStreamingResponse:
    def __init__(self, species_list: SpeciesListResource) -> None:
        self._species_list = species_list

        self.list = to_streamed_response_wrapper(
            species_list.list,
        )


class AsyncSpeciesListResourceWithStreamingResponse:
    def __init__(self, species_list: AsyncSpeciesListResource) -> None:
        self._species_list = species_list

        self.list = async_to_streamed_response_wrapper(
            species_list.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .data import (
    DataResource,
    AsyncDataResource,
    DataResourceWithRawResponse,
    AsyncDataResourceWithRawResponse,
    DataResourceWithStreamingResponse,
    AsyncDataResourceWithStreamingResponse,
)
from .observations import (
    ObservationsResource,
    AsyncObservationsResource,
    ObservationsResourceWithRawResponse,
    AsyncObservationsResourceWithRawResponse,
    ObservationsResourceWithStreamingResponse,
    AsyncObservationsResourceWithStreamingResponse,
)

__all__ = [
    "ObservationsResource",
    "AsyncObservationsResource",
    "ObservationsResourceWithRawResponse",
    "AsyncObservationsResourceWithRawResponse",
    "ObservationsResourceWithStreamingResponse",
    "AsyncObservationsResourceWithStreamingResponse",
    "DataResource",
    "AsyncDataResource",
    "DataResourceWithRawResponse",
    "AsyncDataResourceWithRawResponse",
    "DataResourceWithStreamingResponse",
    "AsyncDataResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["SpeciesGroupListParams"]


class SpeciesGroupListParams(TypedDict, total=False):
    group_name_locale: Annotated[str, PropertyInfo(alias="groupNameLocale")]
    """Locale for species group names.

    English names are returned for any non-listed locale or any non-translated group
    name.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["HotspotListParams"]


class HotspotListParams(TypedDict, total=False):
    back: int
    """The number of days back to fetch hotspots."""

    fmt: Literal["csv", "json"]
    """Fetch the records in CSV or JSON format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .nearest import (
    NearestResource,
    AsyncNearestResource,
    NearestResourceWithRawResponse,
    AsyncNearestResourceWithRawResponse,
    NearestResourceWithStreamingResponse,
    AsyncNearestResourceWithStreamingResponse,
)
from .geo_species import (
    GeoSpeciesResource,
    AsyncGeoSpeciesResource,
    GeoSpeciesResourceWithRawResponse,
    AsyncGeoSpeciesResourceWithRawResponse,
    GeoSpeciesResourceWithStreamingResponse,
    AsyncGeoSpeciesResourceWithStreamingResponse,
)

__all__ = [
    "GeoSpeciesResource",
    "AsyncGeoSpeciesResource",
    "GeoSpeciesResourceWithRawResponse",
    "AsyncGeoSpeciesResourceWithRawResponse",
    "GeoSpeciesResourceWithStreamingResponse",
    "AsyncGeoSpeciesResourceWithStreamingResponse",
    "NearestResource",
    "AsyncNearestResource",
    "NearestResourceWithRawResponse",
    "AsyncNearestResourceWithRawResponse",
    "NearestResourceWithStreamingResponse",
    "AsyncNearestResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .info import (
    InfoResource,
    AsyncInfoResource,
    InfoResourceWithRawResponse,
    AsyncInfoResourceWithRawResponse,
    InfoResourceWithStreamingResponse,
    AsyncInfoResourceWithStreamingResponse,
)
from .list import (
    ListResource,
    AsyncListResource,
    ListResourceWithRawResponse,
    AsyncListResourceWithRawResponse,
    ListResourceWithStreamingResponse,
    AsyncListResourceWithStreamingResponse,
)
from .region import (
    RegionResource,
    AsyncRegionResource,
    RegionResourceWithRawResponse,
    AsyncRegionResourceWithRawResponse,
    RegionResourceWithStreamingResponse,
    AsyncRegionResourceWithStreamingResponse,
)
from .adjacent import (
    AdjacentResource,
    AsyncAdjacentResource,
    AdjacentResourceWithRawResponse,
    AsyncAdjacentResourceWithRawResponse,
    AdjacentResourceWithStreamingResponse,
    AsyncAdjacentResourceWithStreamingResponse,
)

__all__ = [
    "AdjacentResource",
    "AsyncAdjacentResource",
    "AdjacentResourceWithRawResponse",
    "AsyncAdjacentResourceWithRawResponse",
    "AdjacentResourceWithStreamingResponse",
    "AsyncAdjacentResourceWithStreamingResponse",
    "InfoResource",
    "AsyncInfoResource",
    "InfoResourceWithRawResponse",
    "AsyncInfoResourceWithRawResponse",
    "InfoResourceWithStreamingResponse",
    "AsyncInfoResourceWithStreamingResponse",
    "ListResource",
    "AsyncListResource",
    "ListResourceWithRawResponse",
    "AsyncListResourceWithRawResponse",
    "ListResourceWithStreamingResponse",
    "AsyncListResourceWithStreamingResponse",
    "RegionResource",
    "AsyncRegionResource",
    "RegionResourceWithRawResponse",
    "AsyncRegionResourceWithRawResponse",
    "RegionResourceWithStreamingResponse",
    "AsyncRegionResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.data.observations.recent import specie_retrieve_params
from .....types.data.observations.recent.specie_retrieve_response import SpecieRetrieveResponse

__all__ = ["SpeciesResource", "AsyncSpeciesResource"]


class SpeciesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpeciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return SpeciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return SpeciesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        species_code: str,
        *,
        region_code: str,
        back: int | NotGiven = NOT_GIVEN,
        hotspot: bool | NotGiven = NOT_GIVEN,
        include_provisional: bool | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        r: List[str] | NotGiven = NOT_GIVEN,
        spp_locale: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpecieRetrieveResponse:
        """
        Get the recent observations, up to 30 days ago, of a particular species in a
        country, region or location. Results include only the most recent observation
        from each location in the region specified.

        #### Notes

        The species code is typically a 6-letter code, e.g. cangoo for Canada Goose. You
        can get complete set of species code from the GET eBird Taxonomy end-point.

        When using the _r_ query parameter set the _regionCode_ URL parameter to an
        empty string.

        Args:
          back: The number of days back to fetch observations.

          hotspot: Only fetch observations from hotspots

          include_provisional: Include observations which have not yet been reviewed.

          max_results: Only fetch this number of observations

          r: Fetch observations from up to 10 locations

          spp_locale: Use this language for species common names

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        if not species_code:
            raise ValueError(f"Expected a non-empty value for `species_code` but received {species_code!r}")
        return self._get(
            f"/data/obs/{region_code}/recent/{species_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "back": back,
                        "hotspot": hotspot,
                        "include_provisional": include_provisional,
                        "max_results": max_results,
                        "r": r,
                        "spp_locale": spp_locale,
                    },
                    specie_retrieve_params.SpecieRetrieveParams,
                ),
            ),
            cast_to=SpecieRetrieveResponse,
        )


class AsyncSpeciesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpeciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncSpeciesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        species_code: str,
        *,
        region_code: str,
        back: int | NotGiven = NOT_GIVEN,
        hotspot: bool | NotGiven = NOT_GIVEN,
        include_provisional: bool | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        r: List[str] | NotGiven = NOT_GIVEN,
        spp_locale: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpecieRetrieveResponse:
        """
        Get the recent observations, up to 30 days ago, of a particular species in a
        country, region or location. Results include only the most recent observation
        from each location in the region specified.

        #### Notes

        The species code is typically a 6-letter code, e.g. cangoo for Canada Goose. You
        can get complete set of species code from the GET eBird Taxonomy end-point.

        When using the _r_ query parameter set the _regionCode_ URL parameter to an
        empty string.

        Args:
          back: The number of days back to fetch observations.

          hotspot: Only fetch observations from hotspots

          include_provisional: Include observations which have not yet been reviewed.

          max_results: Only fetch this number of observations

          r: Fetch observations from up to 10 locations

          spp_locale: Use this language for species common names

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        if not species_code:
            raise ValueError(f"Expected a non-empty value for `species_code` but received {species_code!r}")
        return await self._get(
            f"/data/obs/{region_code}/recent/{species_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "back": back,
                        "hotspot": hotspot,
                        "include_provisional": include_provisional,
                        "max_results": max_results,
                        "r": r,
                        "spp_locale": spp_locale,
                    },
                    specie_retrieve_params.SpecieRetrieveParams,
                ),
            ),
            cast_to=SpecieRetrieveResponse,
        )


class SpeciesResourceWithRawResponse:
    def __init__(self, species: SpeciesResource) -> None:
        self._species = species

        self.retrieve = to_raw_response_wrapper(
            species.retrieve,
        )


class AsyncSpeciesResourceWithRawResponse:
    def __init__(self, species: AsyncSpeciesResource) -> None:
        self._species = species

        self.retrieve = async_to_raw_response_wrapper(
            species.retrieve,
        )


class SpeciesResourceWithStreamingResponse:
    def __init__(self, species: SpeciesResource) -> None:
        self._species = species

        self.retrieve = to_streamed_response_wrapper(
            species.retrieve,
        )


class AsyncSpeciesResourceWithStreamingResponse:
    def __init__(self, species: AsyncSpeciesResource) -> None:
        self._species = species

        self.retrieve = async_to_streamed_response_wrapper(
            species.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .list_retrieve_params import ListRetrieveParams as ListRetrieveParams
from .list_retrieve_response import ListRetrieveResponse as ListRetrieveResponse
from .stat_retrieve_response import StatRetrieveResponse as StatRetrieveResponse
from .top100_retrieve_params import Top100RetrieveParams as Top100RetrieveParams
from .checklist_view_response import ChecklistViewResponse as ChecklistViewResponse
from .top100_retrieve_response import Top100RetrieveResponse as Top100RetrieveResponse
from .species_list_list_response import SpeciesListListResponse as SpeciesListListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ....._compat import cached_property
from .geo_species import (
    GeoSpeciesResource,
    AsyncGeoSpeciesResource,
    GeoSpeciesResourceWithRawResponse,
    AsyncGeoSpeciesResourceWithRawResponse,
    GeoSpeciesResourceWithStreamingResponse,
    AsyncGeoSpeciesResourceWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["NearestResource", "AsyncNearestResource"]


class NearestResource(SyncAPIResource):
    @cached_property
    def geo_species(self) -> GeoSpeciesResource:
        return GeoSpeciesResource(self._client)

    @cached_property
    def with_raw_response(self) -> NearestResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return NearestResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NearestResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return NearestResourceWithStreamingResponse(self)


class AsyncNearestResource(AsyncAPIResource):
    @cached_property
    def geo_species(self) -> AsyncGeoSpeciesResource:
        return AsyncGeoSpeciesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncNearestResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNearestResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNearestResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncNearestResourceWithStreamingResponse(self)


class NearestResourceWithRawResponse:
    def __init__(self, nearest: NearestResource) -> None:
        self._nearest = nearest

    @cached_property
    def geo_species(self) -> GeoSpeciesResourceWithRawResponse:
        return GeoSpeciesResourceWithRawResponse(self._nearest.geo_species)


class AsyncNearestResourceWithRawResponse:
    def __init__(self, nearest: AsyncNearestResource) -> None:
        self._nearest = nearest

    @cached_property
    def geo_species(self) -> AsyncGeoSpeciesResourceWithRawResponse:
        return AsyncGeoSpeciesResourceWithRawResponse(self._nearest.geo_species)


class NearestResourceWithStreamingResponse:
    def __init__(self, nearest: NearestResource) -> None:
        self._nearest = nearest

    @cached_property
    def geo_species(self) -> GeoSpeciesResourceWithStreamingResponse:
        return GeoSpeciesResourceWithStreamingResponse(self._nearest.geo_species)


class AsyncNearestResourceWithStreamingResponse:
    def __init__(self, nearest: AsyncNearestResource) -> None:
        self._nearest = nearest

    @cached_property
    def geo_species(self) -> AsyncGeoSpeciesResourceWithStreamingResponse:
        return AsyncGeoSpeciesResourceWithStreamingResponse(self._nearest.geo_species)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ref.taxonomy import ebird_retrieve_params
from ....types.ref.taxonomy.ebird_retrieve_response import EbirdRetrieveResponse

__all__ = ["EbirdResource", "AsyncEbirdResource"]


class EbirdResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EbirdResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return EbirdResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EbirdResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return EbirdResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        cat: str | NotGiven = NOT_GIVEN,
        fmt: Literal["csv", "json"] | NotGiven = NOT_GIVEN,
        locale: str | NotGiven = NOT_GIVEN,
        species: str | NotGiven = NOT_GIVEN,
        version: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EbirdRetrieveResponse:
        """Get the taxonomy used by eBird.

        #### Notes Each entry in the taxonomy contains a
        species code for example, barswa for Barn Swallow. You can download the taxonomy
        for selected species using the _species_ query parameter with a comma separating
        each code. Otherwise the full taxonomy is downloaded.

        Args:
          cat: Only fetch records from these taxonomic categories.

          fmt: Fetch the records in CSV or JSON format.

          locale: Use this language for common names.

          species: Only fetch records for these species.

          version: Fetch a specific version of the taxonomy.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/ref/taxonomy/ebird",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cat": cat,
                        "fmt": fmt,
                        "locale": locale,
                        "species": species,
                        "version": version,
                    },
                    ebird_retrieve_params.EbirdRetrieveParams,
                ),
            ),
            cast_to=EbirdRetrieveResponse,
        )


class AsyncEbirdResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEbirdResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEbirdResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEbirdResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncEbirdResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        cat: str | NotGiven = NOT_GIVEN,
        fmt: Literal["csv", "json"] | NotGiven = NOT_GIVEN,
        locale: str | NotGiven = NOT_GIVEN,
        species: str | NotGiven = NOT_GIVEN,
        version: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EbirdRetrieveResponse:
        """Get the taxonomy used by eBird.

        #### Notes Each entry in the taxonomy contains a
        species code for example, barswa for Barn Swallow. You can download the taxonomy
        for selected species using the _species_ query parameter with a comma separating
        each code. Otherwise the full taxonomy is downloaded.

        Args:
          cat: Only fetch records from these taxonomic categories.

          fmt: Fetch the records in CSV or JSON format.

          locale: Use this language for common names.

          species: Only fetch records for these species.

          version: Fetch a specific version of the taxonomy.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/ref/taxonomy/ebird",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cat": cat,
                        "fmt": fmt,
                        "locale": locale,
                        "species": species,
                        "version": version,
                    },
                    ebird_retrieve_params.EbirdRetrieveParams,
                ),
            ),
            cast_to=EbirdRetrieveResponse,
        )


class EbirdResourceWithRawResponse:
    def __init__(self, ebird: EbirdResource) -> None:
        self._ebird = ebird

        self.retrieve = to_raw_response_wrapper(
            ebird.retrieve,
        )


class AsyncEbirdResourceWithRawResponse:
    def __init__(self, ebird: AsyncEbirdResource) -> None:
        self._ebird = ebird

        self.retrieve = async_to_raw_response_wrapper(
            ebird.retrieve,
        )


class EbirdResourceWithStreamingResponse:
    def __init__(self, ebird: EbirdResource) -> None:
        self._ebird = ebird

        self.retrieve = to_streamed_response_wrapper(
            ebird.retrieve,
        )


class AsyncEbirdResourceWithStreamingResponse:
    def __init__(self, ebird: AsyncEbirdResource) -> None:
        self._ebird = ebird

        self.retrieve = async_to_streamed_response_wrapper(
            ebird.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .recent import (
    RecentResource,
    AsyncRecentResource,
    RecentResourceWithRawResponse,
    AsyncRecentResourceWithRawResponse,
    RecentResourceWithStreamingResponse,
    AsyncRecentResourceWithStreamingResponse,
)
from .notable import (
    NotableResource,
    AsyncNotableResource,
    NotableResourceWithRawResponse,
    AsyncNotableResourceWithRawResponse,
    NotableResourceWithStreamingResponse,
    AsyncNotableResourceWithStreamingResponse,
)
from .species import (
    SpeciesResource,
    AsyncSpeciesResource,
    SpeciesResourceWithRawResponse,
    AsyncSpeciesResourceWithRawResponse,
    SpeciesResourceWithStreamingResponse,
    AsyncSpeciesResourceWithStreamingResponse,
)

__all__ = [
    "SpeciesResource",
    "AsyncSpeciesResource",
    "SpeciesResourceWithRawResponse",
    "AsyncSpeciesResourceWithRawResponse",
    "SpeciesResourceWithStreamingResponse",
    "AsyncSpeciesResourceWithStreamingResponse",
    "NotableResource",
    "AsyncNotableResource",
    "NotableResourceWithRawResponse",
    "AsyncNotableResourceWithRawResponse",
    "NotableResourceWithStreamingResponse",
    "AsyncNotableResourceWithStreamingResponse",
    "RecentResource",
    "AsyncRecentResource",
    "RecentResourceWithRawResponse",
    "AsyncRecentResourceWithRawResponse",
    "RecentResourceWithStreamingResponse",
    "AsyncRecentResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .ebird import (
    EbirdResource,
    AsyncEbirdResource,
    EbirdResourceWithRawResponse,
    AsyncEbirdResourceWithRawResponse,
    EbirdResourceWithStreamingResponse,
    AsyncEbirdResourceWithStreamingResponse,
)
from .forms import (
    FormsResource,
    AsyncFormsResource,
    FormsResourceWithRawResponse,
    AsyncFormsResourceWithRawResponse,
    FormsResourceWithStreamingResponse,
    AsyncFormsResourceWithStreamingResponse,
)
from .locales import (
    LocalesResource,
    AsyncLocalesResource,
    LocalesResourceWithRawResponse,
    AsyncLocalesResourceWithRawResponse,
    LocalesResourceWithStreamingResponse,
    AsyncLocalesResourceWithStreamingResponse,
)
from .taxonomy import (
    TaxonomyResource,
    AsyncTaxonomyResource,
    TaxonomyResourceWithRawResponse,
    AsyncTaxonomyResourceWithRawResponse,
    TaxonomyResourceWithStreamingResponse,
    AsyncTaxonomyResourceWithStreamingResponse,
)
from .versions import (
    VersionsResource,
    AsyncVersionsResource,
    VersionsResourceWithRawResponse,
    AsyncVersionsResourceWithRawResponse,
    VersionsResourceWithStreamingResponse,
    AsyncVersionsResourceWithStreamingResponse,
)
from .species_groups import (
    SpeciesGroupsResource,
    AsyncSpeciesGroupsResource,
    SpeciesGroupsResourceWithRawResponse,
    AsyncSpeciesGroupsResourceWithRawResponse,
    SpeciesGroupsResourceWithStreamingResponse,
    AsyncSpeciesGroupsResourceWithStreamingResponse,
)

__all__ = [
    "EbirdResource",
    "AsyncEbirdResource",
    "EbirdResourceWithRawResponse",
    "AsyncEbirdResourceWithRawResponse",
    "EbirdResourceWithStreamingResponse",
    "AsyncEbirdResourceWithStreamingResponse",
    "FormsResource",
    "AsyncFormsResource",
    "FormsResourceWithRawResponse",
    "AsyncFormsResourceWithRawResponse",
    "FormsResourceWithStreamingResponse",
    "AsyncFormsResourceWithStreamingResponse",
    "LocalesResource",
    "AsyncLocalesResource",
    "LocalesResourceWithRawResponse",
    "AsyncLocalesResourceWithRawResponse",
    "LocalesResourceWithStreamingResponse",
    "AsyncLocalesResourceWithStreamingResponse",
    "VersionsResource",
    "AsyncVersionsResource",
    "VersionsResourceWithRawResponse",
    "AsyncVersionsResourceWithRawResponse",
    "VersionsResourceWithStreamingResponse",
    "AsyncVersionsResourceWithStreamingResponse",
    "SpeciesGroupsResource",
    "AsyncSpeciesGroupsResource",
    "SpeciesGroupsResourceWithRawResponse",
    "AsyncSpeciesGroupsResourceWithRawResponse",
    "SpeciesGroupsResourceWithStreamingResponse",
    "AsyncSpeciesGroupsResourceWithStreamingResponse",
    "TaxonomyResource",
    "AsyncTaxonomyResource",
    "TaxonomyResourceWithRawResponse",
    "AsyncTaxonomyResourceWithRawResponse",
    "TaxonomyResourceWithStreamingResponse",
    "AsyncTaxonomyResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .historical import (
    HistoricalResource,
    AsyncHistoricalResource,
    HistoricalResourceWithRawResponse,
    AsyncHistoricalResourceWithRawResponse,
    HistoricalResourceWithStreamingResponse,
    AsyncHistoricalResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.product import list_retrieve_params
from ....types.product.list_retrieve_response import ListRetrieveResponse

__all__ = ["ListsResource", "AsyncListsResource"]


class ListsResource(SyncAPIResource):
    @cached_property
    def historical(self) -> HistoricalResource:
        return HistoricalResource(self._client)

    @cached_property
    def with_raw_response(self) -> ListsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return ListsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ListsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return ListsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        region_code: str,
        *,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ListRetrieveResponse:
        """
        Get information on the most recently submitted checklists for a region.

        Args:
          max_results: Only fetch this number of checklists.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return self._get(
            f"/product/lists/{region_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"max_results": max_results}, list_retrieve_params.ListRetrieveParams),
            ),
            cast_to=ListRetrieveResponse,
        )


class AsyncListsResource(AsyncAPIResource):
    @cached_property
    def historical(self) -> AsyncHistoricalResource:
        return AsyncHistoricalResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncListsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncListsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncListsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncListsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        region_code: str,
        *,
        max_results: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ListRetrieveResponse:
        """
        Get information on the most recently submitted checklists for a region.

        Args:
          max_results: Only fetch this number of checklists.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return await self._get(
            f"/product/lists/{region_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"max_results": max_results}, list_retrieve_params.ListRetrieveParams
                ),
            ),
            cast_to=ListRetrieveResponse,
        )


class ListsResourceWithRawResponse:
    def __init__(self, lists: ListsResource) -> None:
        self._lists = lists

        self.retrieve = to_raw_response_wrapper(
            lists.retrieve,
        )

    @cached_property
    def historical(self) -> HistoricalResourceWithRawResponse:
        return HistoricalResourceWithRawResponse(self._lists.historical)


class AsyncListsResourceWithRawResponse:
    def __init__(self, lists: AsyncListsResource) -> None:
        self._lists = lists

        self.retrieve = async_to_raw_response_wrapper(
            lists.retrieve,
        )

    @cached_property
    def historical(self) -> AsyncHistoricalResourceWithRawResponse:
        return AsyncHistoricalResourceWithRawResponse(self._lists.historical)


class ListsResourceWithStreamingResponse:
    def __init__(self, lists: ListsResource) -> None:
        self._lists = lists

        self.retrieve = to_streamed_response_wrapper(
            lists.retrieve,
        )

    @cached_property
    def historical(self) -> HistoricalResourceWithStreamingResponse:
        return HistoricalResourceWithStreamingResponse(self._lists.historical)


class AsyncListsResourceWithStreamingResponse:
    def __init__(self, lists: AsyncListsResource) -> None:
        self._lists = lists

        self.retrieve = async_to_streamed_response_wrapper(
            lists.retrieve,
        )

    @cached_property
    def historical(self) -> AsyncHistoricalResourceWithStreamingResponse:
        return AsyncHistoricalResourceWithStreamingResponse(self._lists.historical)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ref.region import info_retrieve_params
from ....types.ref.region.info_retrieve_response import InfoRetrieveResponse

__all__ = ["InfoResource", "AsyncInfoResource"]


class InfoResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InfoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return InfoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InfoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return InfoResourceWithStreamingResponse(self)

    def retrieve(
        self,
        region_code: str,
        *,
        delim: str | NotGiven = NOT_GIVEN,
        region_name_format: Literal["detailed", "detailednoqual", "full", "namequal", "nameonly", "revdetailed"]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InfoRetrieveResponse:
        """
        Get information on the name and geographical area covered by a region.

        #### Notes

        Taking Madison County, New York, USA (location code US-NY-053) as an example the
        various values for the regionNameFormat query parameter work as follows:

        | Value          | Description                                | Result                           |
        | -------------- | ------------------------------------------ | -------------------------------- |
        | detailed       | return a detailed description              | Madison County, New York, US     |
        | detailednoqual | return the name to the subnational1 level  | Madison, New York                |
        | full           | return the full description                | Madison, New York, United States |
        | namequal       | return the qualified name                  | Madison County                   |
        | nameonly       | return only the name of the region         | Madison                          |
        | revdetailed    | return the detailed description in reverse | US, New York, Madison County     |

        Args:
          delim: The characters used to separate elements in the name.

          region_name_format: Control how the name is displayed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return self._get(
            f"/ref/region/info/{region_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "delim": delim,
                        "region_name_format": region_name_format,
                    },
                    info_retrieve_params.InfoRetrieveParams,
                ),
            ),
            cast_to=InfoRetrieveResponse,
        )


class AsyncInfoResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInfoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInfoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInfoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncInfoResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        region_code: str,
        *,
        delim: str | NotGiven = NOT_GIVEN,
        region_name_format: Literal["detailed", "detailednoqual", "full", "namequal", "nameonly", "revdetailed"]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InfoRetrieveResponse:
        """
        Get information on the name and geographical area covered by a region.

        #### Notes

        Taking Madison County, New York, USA (location code US-NY-053) as an example the
        various values for the regionNameFormat query parameter work as follows:

        | Value          | Description                                | Result                           |
        | -------------- | ------------------------------------------ | -------------------------------- |
        | detailed       | return a detailed description              | Madison County, New York, US     |
        | detailednoqual | return the name to the subnational1 level  | Madison, New York                |
        | full           | return the full description                | Madison, New York, United States |
        | namequal       | return the qualified name                  | Madison County                   |
        | nameonly       | return only the name of the region         | Madison                          |
        | revdetailed    | return the detailed description in reverse | US, New York, Madison County     |

        Args:
          delim: The characters used to separate elements in the name.

          region_name_format: Control how the name is displayed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not region_code:
            raise ValueError(f"Expected a non-empty value for `region_code` but received {region_code!r}")
        return await self._get(
            f"/ref/region/info/{region_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "delim": delim,
                        "region_name_format": region_name_format,
                    },
                    info_retrieve_params.InfoRetrieveParams,
                ),
            ),
            cast_to=InfoRetrieveResponse,
        )


class InfoResourceWithRawResponse:
    def __init__(self, info: InfoResource) -> None:
        self._info = info

        self.retrieve = to_raw_response_wrapper(
            info.retrieve,
        )


class AsyncInfoResourceWithRawResponse:
    def __init__(self, info: AsyncInfoResource) -> None:
        self._info = info

        self.retrieve = async_to_raw_response_wrapper(
            info.retrieve,
        )


class InfoResourceWithStreamingResponse:
    def __init__(self, info: InfoResource) -> None:
        self._info = info

        self.retrieve = to_streamed_response_wrapper(
            info.retrieve,
        )


class AsyncInfoResourceWithStreamingResponse:
    def __init__(self, info: AsyncInfoResource) -> None:
        self._info = info

        self.retrieve = async_to_streamed_response_wrapper(
            info.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["Observation"]


class Observation(BaseModel):
    id: Optional[int] = None

    com_name: Optional[str] = FieldInfo(alias="comName", default=None)

    firstname: Optional[str] = None

    how_many: Optional[int] = FieldInfo(alias="howMany", default=None)

    lastname: Optional[str] = None

    lat: Optional[float] = None

    lng: Optional[float] = None

    location_private: Optional[bool] = FieldInfo(alias="locationPrivate", default=None)

    loc_id: Optional[str] = FieldInfo(alias="locId", default=None)

    loc_name: Optional[str] = FieldInfo(alias="locName", default=None)

    obs_dt: Optional[str] = FieldInfo(alias="obsDt", default=None)

    obs_reviewed: Optional[bool] = FieldInfo(alias="obsReviewed", default=None)

    obs_valid: Optional[bool] = FieldInfo(alias="obsValid", default=None)

    sci_name: Optional[str] = FieldInfo(alias="sciName", default=None)

    species_code: Optional[str] = FieldInfo(alias="speciesCode", default=None)

    sub_id: Optional[str] = FieldInfo(alias="subId", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .geo_retrieve_params import GeoRetrieveParams as GeoRetrieveParams
from .geo_retrieve_response import GeoRetrieveResponse as GeoRetrieveResponse
from .info_retrieve_response import InfoRetrieveResponse as InfoRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["Top100RetrieveParams"]


class Top100RetrieveParams(TypedDict, total=False):
    region_code: Required[Annotated[str, PropertyInfo(alias="regionCode")]]

    y: Required[int]

    m: Required[int]

    max_results: Annotated[int, PropertyInfo(alias="maxResults")]
    """Only fetch this number of contributors."""

    ranked_by: Annotated[Literal["spp", "cl"], PropertyInfo(alias="rankedBy")]
    """Order by number of complete checklists (cl) or by number of species seen (spp)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ListListParams"]


class ListListParams(TypedDict, total=False):
    region_type: Required[Annotated[str, PropertyInfo(alias="regionType")]]

    fmt: Literal["csv", "json"]
    """Fetch the records in CSV or JSON format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .ref import (
    RefResource,
    AsyncRefResource,
    RefResourceWithRawResponse,
    AsyncRefResourceWithRawResponse,
    RefResourceWithStreamingResponse,
    AsyncRefResourceWithStreamingResponse,
)
from .region import (
    RegionResource,
    AsyncRegionResource,
    RegionResourceWithRawResponse,
    AsyncRegionResourceWithRawResponse,
    RegionResourceWithStreamingResponse,
    AsyncRegionResourceWithStreamingResponse,
)
from .hotspot import (
    HotspotResource,
    AsyncHotspotResource,
    HotspotResourceWithRawResponse,
    AsyncHotspotResourceWithRawResponse,
    HotspotResourceWithStreamingResponse,
    AsyncHotspotResourceWithStreamingResponse,
)
from .taxonomy import (
    TaxonomyResource,
    AsyncTaxonomyResource,
    TaxonomyResourceWithRawResponse,
    AsyncTaxonomyResourceWithRawResponse,
    TaxonomyResourceWithStreamingResponse,
    AsyncTaxonomyResourceWithStreamingResponse,
)

__all__ = [
    "RegionResource",
    "AsyncRegionResource",
    "RegionResourceWithRawResponse",
    "AsyncRegionResourceWithRawResponse",
    "RegionResourceWithStreamingResponse",
    "AsyncRegionResourceWithStreamingResponse",
    "HotspotResource",
    "AsyncHotspotResource",
    "HotspotResourceWithRawResponse",
    "AsyncHotspotResourceWithRawResponse",
    "HotspotResourceWithStreamingResponse",
    "AsyncHotspotResourceWithStreamingResponse",
    "TaxonomyResource",
    "AsyncTaxonomyResource",
    "TaxonomyResourceWithRawResponse",
    "AsyncTaxonomyResourceWithRawResponse",
    "TaxonomyResourceWithStreamingResponse",
    "AsyncTaxonomyResourceWithStreamingResponse",
    "RefResource",
    "AsyncRefResource",
    "RefResourceWithRawResponse",
    "AsyncRefResourceWithRawResponse",
    "RefResourceWithStreamingResponse",
    "AsyncRefResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["InfoRetrieveResponse"]


class InfoRetrieveResponse(BaseModel):
    country_code: Optional[str] = FieldInfo(alias="countryCode", default=None)

    country_name: Optional[str] = FieldInfo(alias="countryName", default=None)

    hierarchical_name: Optional[str] = FieldInfo(alias="hierarchicalName", default=None)

    is_hotspot: Optional[bool] = FieldInfo(alias="isHotspot", default=None)

    lat: Optional[float] = None

    latitude: Optional[float] = None

    lng: Optional[float] = None

    loc_id: Optional[str] = FieldInfo(alias="locId", default=None)

    loc_name: Optional[str] = FieldInfo(alias="locName", default=None)

    longitude: Optional[float] = None

    name: Optional[str] = None

    subnational1_code: Optional[str] = FieldInfo(alias="subnational1Code", default=None)

    subnational1_name: Optional[str] = FieldInfo(alias="subnational1Name", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["EbirdRetrieveResponse", "EbirdRetrieveResponseItem"]


class EbirdRetrieveResponseItem(BaseModel):
    banding_codes: Optional[List[str]] = FieldInfo(alias="bandingCodes", default=None)

    category: Optional[str] = None

    com_name: Optional[str] = FieldInfo(alias="comName", default=None)

    com_name_codes: Optional[List[str]] = FieldInfo(alias="comNameCodes", default=None)

    family_code: Optional[str] = FieldInfo(alias="familyCode", default=None)

    family_com_name: Optional[str] = FieldInfo(alias="familyComName", default=None)

    family_sci_name: Optional[str] = FieldInfo(alias="familySciName", default=None)

    order: Optional[str] = None

    sci_name: Optional[str] = FieldInfo(alias="sciName", default=None)

    sci_name_codes: Optional[List[str]] = FieldInfo(alias="sciNameCodes", default=None)

    species_code: Optional[str] = FieldInfo(alias="speciesCode", default=None)

    taxon_order: Optional[int] = FieldInfo(alias="taxonOrder", default=None)


EbirdRetrieveResponse: TypeAlias = List[EbirdRetrieveResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .form_list_response import FormListResponse as FormListResponse
from .locale_list_response import LocaleListResponse as LocaleListResponse
from .ebird_retrieve_params import EbirdRetrieveParams as EbirdRetrieveParams
from .version_list_response import VersionListResponse as VersionListResponse
from .ebird_retrieve_response import EbirdRetrieveResponse as EbirdRetrieveResponse
from .species_group_list_params import SpeciesGroupListParams as SpeciesGroupListParams
from .species_group_list_response import SpeciesGroupListResponse as SpeciesGroupListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["HistoricListParams"]


class HistoricListParams(TypedDict, total=False):
    region_code: Required[Annotated[str, PropertyInfo(alias="regionCode")]]

    y: Required[int]

    m: Required[int]

    cat: Literal["species", "slash", "issf", "spuh", "hybrid", "domestic", "form", "intergrade"]
    """Only fetch observations from these taxonomic categories"""

    detail: Literal["simple", "full"]
    """Include a subset (simple), or all (full), of the fields available."""

    hotspot: bool
    """Only fetch observations from hotspots"""

    include_provisional: Annotated[bool, PropertyInfo(alias="includeProvisional")]
    """Include observations which have not yet been reviewed."""

    max_results: Annotated[int, PropertyInfo(alias="maxResults")]
    """Only fetch this number of observations"""

    r: List[str]
    """Fetch observations from up to 50 locations"""

    rank: Literal["mrec", "create"]
    """Include latest observation of the day, or the first added"""

    spp_locale: Annotated[str, PropertyInfo(alias="sppLocale")]
    """Use this language for species common names"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ListRetrieveParams"]


class ListRetrieveParams(TypedDict, total=False):
    max_results: Annotated[int, PropertyInfo(alias="maxResults")]
    """Only fetch this number of checklists."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .historical_retrieve_params import HistoricalRetrieveParams as HistoricalRetrieveParams
from .historical_retrieve_response import HistoricalRetrieveResponse as HistoricalRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ....observation import Observation

__all__ = ["SpecieListResponse"]

SpecieListResponse: TypeAlias = List[Observation]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .region.region import (
    RegionResource,
    AsyncRegionResource,
    RegionResourceWithRawResponse,
    AsyncRegionResourceWithRawResponse,
    RegionResourceWithStreamingResponse,
    AsyncRegionResourceWithStreamingResponse,
)
from .hotspot.hotspot import (
    HotspotResource,
    AsyncHotspotResource,
    HotspotResourceWithRawResponse,
    AsyncHotspotResourceWithRawResponse,
    HotspotResourceWithStreamingResponse,
    AsyncHotspotResourceWithStreamingResponse,
)
from .taxonomy.taxonomy import (
    TaxonomyResource,
    AsyncTaxonomyResource,
    TaxonomyResourceWithRawResponse,
    AsyncTaxonomyResourceWithRawResponse,
    TaxonomyResourceWithStreamingResponse,
    AsyncTaxonomyResourceWithStreamingResponse,
)

__all__ = ["RefResource", "AsyncRefResource"]


class RefResource(SyncAPIResource):
    @cached_property
    def region(self) -> RegionResource:
        return RegionResource(self._client)

    @cached_property
    def hotspot(self) -> HotspotResource:
        return HotspotResource(self._client)

    @cached_property
    def taxonomy(self) -> TaxonomyResource:
        return TaxonomyResource(self._client)

    @cached_property
    def with_raw_response(self) -> RefResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return RefResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RefResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return RefResourceWithStreamingResponse(self)


class AsyncRefResource(AsyncAPIResource):
    @cached_property
    def region(self) -> AsyncRegionResource:
        return AsyncRegionResource(self._client)

    @cached_property
    def hotspot(self) -> AsyncHotspotResource:
        return AsyncHotspotResource(self._client)

    @cached_property
    def taxonomy(self) -> AsyncTaxonomyResource:
        return AsyncTaxonomyResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRefResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRefResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRefResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncRefResourceWithStreamingResponse(self)


class RefResourceWithRawResponse:
    def __init__(self, ref: RefResource) -> None:
        self._ref = ref

    @cached_property
    def region(self) -> RegionResourceWithRawResponse:
        return RegionResourceWithRawResponse(self._ref.region)

    @cached_property
    def hotspot(self) -> HotspotResourceWithRawResponse:
        return HotspotResourceWithRawResponse(self._ref.hotspot)

    @cached_property
    def taxonomy(self) -> TaxonomyResourceWithRawResponse:
        return TaxonomyResourceWithRawResponse(self._ref.taxonomy)


class AsyncRefResourceWithRawResponse:
    def __init__(self, ref: AsyncRefResource) -> None:
        self._ref = ref

    @cached_property
    def region(self) -> AsyncRegionResourceWithRawResponse:
        return AsyncRegionResourceWithRawResponse(self._ref.region)

    @cached_property
    def hotspot(self) -> AsyncHotspotResourceWithRawResponse:
        return AsyncHotspotResourceWithRawResponse(self._ref.hotspot)

    @cached_property
    def taxonomy(self) -> AsyncTaxonomyResourceWithRawResponse:
        return AsyncTaxonomyResourceWithRawResponse(self._ref.taxonomy)


class RefResourceWithStreamingResponse:
    def __init__(self, ref: RefResource) -> None:
        self._ref = ref

    @cached_property
    def region(self) -> RegionResourceWithStreamingResponse:
        return RegionResourceWithStreamingResponse(self._ref.region)

    @cached_property
    def hotspot(self) -> HotspotResourceWithStreamingResponse:
        return HotspotResourceWithStreamingResponse(self._ref.hotspot)

    @cached_property
    def taxonomy(self) -> TaxonomyResourceWithStreamingResponse:
        return TaxonomyResourceWithStreamingResponse(self._ref.taxonomy)


class AsyncRefResourceWithStreamingResponse:
    def __init__(self, ref: AsyncRefResource) -> None:
        self._ref = ref

    @cached_property
    def region(self) -> AsyncRegionResourceWithStreamingResponse:
        return AsyncRegionResourceWithStreamingResponse(self._ref.region)

    @cached_property
    def hotspot(self) -> AsyncHotspotResourceWithStreamingResponse:
        return AsyncHotspotResourceWithStreamingResponse(self._ref.hotspot)

    @cached_property
    def taxonomy(self) -> AsyncTaxonomyResourceWithStreamingResponse:
        return AsyncTaxonomyResourceWithStreamingResponse(self._ref.taxonomy)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .geo_specie_list_params import GeoSpecieListParams as GeoSpecieListParams
from .geo_specie_list_response import GeoSpecieListResponse as GeoSpecieListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["EbirdRetrieveParams"]


class EbirdRetrieveParams(TypedDict, total=False):
    cat: str
    """Only fetch records from these taxonomic categories."""

    fmt: Literal["csv", "json"]
    """Fetch the records in CSV or JSON format."""

    locale: str
    """Use this language for common names."""

    species: str
    """Only fetch records for these species."""

    version: str
    """Fetch a specific version of the taxonomy."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ref.hotspot.info_retrieve_response import InfoRetrieveResponse

__all__ = ["InfoResource", "AsyncInfoResource"]


class InfoResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InfoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return InfoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InfoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return InfoResourceWithStreamingResponse(self)

    def retrieve(
        self,
        loc_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InfoRetrieveResponse:
        """Get information on the location of a hotspot.

        #### Notes This API call only
        works for hotspots. If you pass the location code for a private location or an
        invalid location code then an HTTP 410 (Gone) error is returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not loc_id:
            raise ValueError(f"Expected a non-empty value for `loc_id` but received {loc_id!r}")
        return self._get(
            f"/ref/hotspot/info/{loc_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InfoRetrieveResponse,
        )


class AsyncInfoResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInfoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInfoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInfoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncInfoResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        loc_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InfoRetrieveResponse:
        """Get information on the location of a hotspot.

        #### Notes This API call only
        works for hotspots. If you pass the location code for a private location or an
        invalid location code then an HTTP 410 (Gone) error is returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not loc_id:
            raise ValueError(f"Expected a non-empty value for `loc_id` but received {loc_id!r}")
        return await self._get(
            f"/ref/hotspot/info/{loc_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InfoRetrieveResponse,
        )


class InfoResourceWithRawResponse:
    def __init__(self, info: InfoResource) -> None:
        self._info = info

        self.retrieve = to_raw_response_wrapper(
            info.retrieve,
        )


class AsyncInfoResourceWithRawResponse:
    def __init__(self, info: AsyncInfoResource) -> None:
        self._info = info

        self.retrieve = async_to_raw_response_wrapper(
            info.retrieve,
        )


class InfoResourceWithStreamingResponse:
    def __init__(self, info: InfoResource) -> None:
        self._info = info

        self.retrieve = to_streamed_response_wrapper(
            info.retrieve,
        )


class AsyncInfoResourceWithStreamingResponse:
    def __init__(self, info: AsyncInfoResource) -> None:
        self._info = info

        self.retrieve = async_to_streamed_response_wrapper(
            info.retrieve,
        )



import os

from phoebe_bird import Phoebe

client = Phoebe(
    api_key=os.environ.get("EBIRD_API_KEY"),
)


def observations() -> None:
    phoebes = client.data.observations.recent.species.retrieve("easpho", region_code="US-NY")

    print(phoebes)

    recent_checklists = client.data.observations.recent.list("US-NY")

    print(recent_checklists)


observations()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .observations.observations import (
    ObservationsResource,
    AsyncObservationsResource,
    ObservationsResourceWithRawResponse,
    AsyncObservationsResourceWithRawResponse,
    ObservationsResourceWithStreamingResponse,
    AsyncObservationsResourceWithStreamingResponse,
)

__all__ = ["DataResource", "AsyncDataResource"]


class DataResource(SyncAPIResource):
    @cached_property
    def observations(self) -> ObservationsResource:
        return ObservationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> DataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return DataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return DataResourceWithStreamingResponse(self)


class AsyncDataResource(AsyncAPIResource):
    @cached_property
    def observations(self) -> AsyncObservationsResource:
        return AsyncObservationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncDataResourceWithStreamingResponse(self)


class DataResourceWithRawResponse:
    def __init__(self, data: DataResource) -> None:
        self._data = data

    @cached_property
    def observations(self) -> ObservationsResourceWithRawResponse:
        return ObservationsResourceWithRawResponse(self._data.observations)


class AsyncDataResourceWithRawResponse:
    def __init__(self, data: AsyncDataResource) -> None:
        self._data = data

    @cached_property
    def observations(self) -> AsyncObservationsResourceWithRawResponse:
        return AsyncObservationsResourceWithRawResponse(self._data.observations)


class DataResourceWithStreamingResponse:
    def __init__(self, data: DataResource) -> None:
        self._data = data

    @cached_property
    def observations(self) -> ObservationsResourceWithStreamingResponse:
        return ObservationsResourceWithStreamingResponse(self._data.observations)


class AsyncDataResourceWithStreamingResponse:
    def __init__(self, data: AsyncDataResource) -> None:
        self._data = data

    @cached_property
    def observations(self) -> AsyncObservationsResourceWithStreamingResponse:
        return AsyncObservationsResourceWithStreamingResponse(self._data.observations)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["HotspotListResponse", "HotspotListResponseItem"]


class HotspotListResponseItem(BaseModel):
    country_code: Optional[str] = FieldInfo(alias="countryCode", default=None)

    lat: Optional[float] = None

    latest_obs_dt: Optional[str] = FieldInfo(alias="latestObsDt", default=None)

    lng: Optional[float] = None

    loc_id: Optional[str] = FieldInfo(alias="locId", default=None)

    loc_name: Optional[str] = FieldInfo(alias="locName", default=None)

    num_species_all_time: Optional[int] = FieldInfo(alias="numSpeciesAllTime", default=None)

    subnational1_code: Optional[str] = FieldInfo(alias="subnational1Code", default=None)

    subnational2_code: Optional[str] = FieldInfo(alias="subnational2Code", default=None)


HotspotListResponse: TypeAlias = List[HotspotListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from .recent.recent import (
    RecentResource,
    AsyncRecentResource,
    RecentResourceWithRawResponse,
    AsyncRecentResourceWithRawResponse,
    RecentResourceWithStreamingResponse,
    AsyncRecentResourceWithStreamingResponse,
)

__all__ = ["GeoResource", "AsyncGeoResource"]


class GeoResource(SyncAPIResource):
    @cached_property
    def recent(self) -> RecentResource:
        return RecentResource(self._client)

    @cached_property
    def with_raw_response(self) -> GeoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return GeoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GeoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return GeoResourceWithStreamingResponse(self)


class AsyncGeoResource(AsyncAPIResource):
    @cached_property
    def recent(self) -> AsyncRecentResource:
        return AsyncRecentResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncGeoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGeoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGeoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncGeoResourceWithStreamingResponse(self)


class GeoResourceWithRawResponse:
    def __init__(self, geo: GeoResource) -> None:
        self._geo = geo

    @cached_property
    def recent(self) -> RecentResourceWithRawResponse:
        return RecentResourceWithRawResponse(self._geo.recent)


class AsyncGeoResourceWithRawResponse:
    def __init__(self, geo: AsyncGeoResource) -> None:
        self._geo = geo

    @cached_property
    def recent(self) -> AsyncRecentResourceWithRawResponse:
        return AsyncRecentResourceWithRawResponse(self._geo.recent)


class GeoResourceWithStreamingResponse:
    def __init__(self, geo: GeoResource) -> None:
        self._geo = geo

    @cached_property
    def recent(self) -> RecentResourceWithStreamingResponse:
        return RecentResourceWithStreamingResponse(self._geo.recent)


class AsyncGeoResourceWithStreamingResponse:
    def __init__(self, geo: AsyncGeoResource) -> None:
        self._geo = geo

    @cached_property
    def recent(self) -> AsyncRecentResourceWithStreamingResponse:
        return AsyncRecentResourceWithStreamingResponse(self._geo.recent)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.product.checklist_view_response import ChecklistViewResponse

__all__ = ["ChecklistResource", "AsyncChecklistResource"]


class ChecklistResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChecklistResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return ChecklistResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChecklistResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return ChecklistResourceWithStreamingResponse(self)

    def view(
        self,
        sub_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChecklistViewResponse:
        """
        Get the details and observations of a checklist.

        #### Notes Do NOT use this to download large amounts of data. You will be banned if you do. In the fields for each observation, the following fields are duplicates or obsolete and will be removed at a future date: _howManyAtleast_, _howManyAtmost_, _hideFlags_, _projId_, _subId_, _subnational1Code_ and _present_.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sub_id:
            raise ValueError(f"Expected a non-empty value for `sub_id` but received {sub_id!r}")
        return self._get(
            f"/product/checklist/view/{sub_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChecklistViewResponse,
        )


class AsyncChecklistResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChecklistResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChecklistResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChecklistResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncChecklistResourceWithStreamingResponse(self)

    async def view(
        self,
        sub_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChecklistViewResponse:
        """
        Get the details and observations of a checklist.

        #### Notes Do NOT use this to download large amounts of data. You will be banned if you do. In the fields for each observation, the following fields are duplicates or obsolete and will be removed at a future date: _howManyAtleast_, _howManyAtmost_, _hideFlags_, _projId_, _subId_, _subnational1Code_ and _present_.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sub_id:
            raise ValueError(f"Expected a non-empty value for `sub_id` but received {sub_id!r}")
        return await self._get(
            f"/product/checklist/view/{sub_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChecklistViewResponse,
        )


class ChecklistResourceWithRawResponse:
    def __init__(self, checklist: ChecklistResource) -> None:
        self._checklist = checklist

        self.view = to_raw_response_wrapper(
            checklist.view,
        )


class AsyncChecklistResourceWithRawResponse:
    def __init__(self, checklist: AsyncChecklistResource) -> None:
        self._checklist = checklist

        self.view = async_to_raw_response_wrapper(
            checklist.view,
        )


class ChecklistResourceWithStreamingResponse:
    def __init__(self, checklist: ChecklistResource) -> None:
        self._checklist = checklist

        self.view = to_streamed_response_wrapper(
            checklist.view,
        )


class AsyncChecklistResourceWithStreamingResponse:
    def __init__(self, checklist: AsyncChecklistResource) -> None:
        self._checklist = checklist

        self.view = async_to_streamed_response_wrapper(
            checklist.view,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import strip_not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ref.taxonomy.locale_list_response import LocaleListResponse

__all__ = ["LocalesResource", "AsyncLocalesResource"]


class LocalesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LocalesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return LocalesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LocalesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return LocalesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        accept_language: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LocaleListResponse:
        """
        Returns the list of supported locale codes and names for species common names,
        with the last time they were updated. Use the accept-language header to get
        translated language names when available.

        NOTE: The locale codes and names are stable but the other fields in this result
        are not yet finalized and should be used with caution.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"Accept-Language": accept_language}), **(extra_headers or {})}
        return self._get(
            "/ref/taxa-locales/ebird",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LocaleListResponse,
        )


class AsyncLocalesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLocalesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLocalesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLocalesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/phoebe-bird/phoebe-python#with_streaming_response
        """
        return AsyncLocalesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        accept_language: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LocaleListResponse:
        """
        Returns the list of supported locale codes and names for species common names,
        with the last time they were updated. Use the accept-language header to get
        translated language names when available.

        NOTE: The locale codes and names are stable but the other fields in this result
        are not yet finalized and should be used with caution.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"Accept-Language": accept_language}), **(extra_headers or {})}
        return await self._get(
            "/ref/taxa-locales/ebird",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LocaleListResponse,
        )


class LocalesResourceWithRawResponse:
    def __init__(self, locales: LocalesResource) -> None:
        self._locales = locales

        self.list = to_raw_response_wrapper(
            locales.list,
        )


class AsyncLocalesResourceWithRawResponse:
    def __init__(self, locales: AsyncLocalesResource) -> None:
        self._locales = locales

        self.list = async_to_raw_response_wrapper(
            locales.list,
        )


class LocalesResourceWithStreamingResponse:
    def __init__(self, locales: LocalesResource) -> None:
        self._locales = locales

        self.list = to_streamed_response_wrapper(
            locales.list,
        )


class AsyncLocalesResourceWithStreamingResponse:
    def __init__(self, locales: AsyncLocalesResource) -> None:
        self._locales = locales

        self.list = async_to_streamed_response_wrapper(
            locales.list,
        )
